@@ -727,19 +727,22 @@ main (int argc, char *argv[])
 	printf (_("Aging information changed.\n"));
     }
 
-#if 0 /* XXX logging */
   if (pw_data->sp.sp_min != pw_data->spn.sp_min)
-    LAUS_LOG(NO_TAG, "chage: password minimum age changed - user=%s, uid=%u, min=%li, oldmin=%li, by=%u",
-	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_min, pw_data->sp.sp_min, uid);
+    sec_log (program, MSG_MINIMUM_AGE,
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid,
+	     pw_data->spn.sp_min, pw_data->sp.sp_min, uid);
   if (pw_data->sp.sp_max != pw_data->spn.sp_max)
-    LAUS_LOG(NO_TAG, "chage: password maximum age changed - user=%s, uid=%u, max=%li, oldmax=%li, by=%u",
-	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_max, pw_data->sp.sp_max, uid);
+    sec_log (program, MSG_MAXIMUM_AGE,
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid,
+	     pw_data->spn.sp_max, pw_data->sp.sp_max, uid);
   if (pw_data->sp.sp_warn != pw_data->spn.sp_warn)
-    LAUS_LOG(NO_TAG, "chage: password warning days changed - user=%s, uid=%u, warn=%li, oldwarn=%li, by=%u",
-	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_warn, pw_data->sp.sp_warn, uid);
+    sec_log (program, MSG_WARNING_DAYS,
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid,
+	     pw_data->spn.sp_warn, pw_data->sp.sp_warn, uid);
   if (pw_data->sp.sp_inact != pw_data->spn.sp_inact)
-    LAUS_LOG(NO_TAG, "chage: password inactive days changed - user=%s, uid=%u, inactive=%li, oldinactive=%li, by=%u",
-	     pw_data->pw.pw_name, pw_data->pw.pw_uid, pw_data->spn.sp_inact, pw_data->sp.sp_inact, uid);
+    sec_log (program, MSG_INACTIVE_DAYS,
+	     pw_data->pw.pw_name, pw_data->pw.pw_uid,
+	     pw_data->spn.sp_inact, pw_data->sp.sp_inact, uid);
 
   if (pw_data->sp.sp_lstchg != pw_data->spn.sp_lstchg)
     {
@@ -747,8 +750,9 @@ main (int argc, char *argv[])
 
       new_lstchg = date2str (pw_data->spn.sp_lstchg * DAY);
       old_lstchg = date2str (pw_data->sp.sp_lstchg * DAY);
-      LAUS_LOG(NO_TAG, "chage: password last change date changed - user=%s, uid=%u, change=%s, oldchange=%s, by=%u",
-	       pw_data->pw.pw_name, pw_data->pw.pw_uid, new_lstchg, old_lstchg, uid);
+      sec_log (program, MSG_LAST_CHANGE_DATE,
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid,
+	       new_lstchg, old_lstchg, uid);
 
       free(new_lstchg);
       free(old_lstchg);
@@ -759,14 +763,13 @@ main (int argc, char *argv[])
 
       new_exp = date2str (pw_data->spn.sp_expire * DAY);
       old_exp = date2str (pw_data->sp.sp_expire * DAY);
-      LAUS_LOG(NO_TAG, "chage: password expiration date changed - user=%s, uid=%u, expire=%s, oldexpire=%s, by=%u",
-	       pw_data->pw.pw_name, pw_data->pw.pw_uid, new_exp, old_exp, uid);
+      sec_log (program, MSG_EXPIRE_DATE,
+	       pw_data->pw.pw_name, pw_data->pw.pw_uid,
+	       new_exp, old_exp, uid);
 
       free(new_exp);
       free(old_exp);
     }
-#endif
-
 
   free_user_t (pw_data);
 
