@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <getopt.h>
 #include <locale.h>
 #include <libintl.h>
@@ -263,8 +262,26 @@ loop_over_group_file (int quiet)
       fclose (input);
       return E_NO_FILE;
     }
-  fchmod (output_fd, group_stat.st_mode);
-  fchown (output_fd, group_stat.st_uid, group_stat.st_gid);
+  if (fchmod (output_fd, group_stat.st_mode) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change mode for `%s': %s\n"),
+	       outputname, strerror (errno));
+      fclose (input);
+      close (output_fd);
+      unlink (outputname);
+      return E_NO_FILE;
+    }
+  if (fchown (output_fd, group_stat.st_uid, group_stat.st_gid) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change owner/group for `%s': %s\n"),
+	       outputname, strerror (errno));
+      fclose (input);
+      close (output_fd);
+      unlink (outputname);
+      return E_NO_FILE;
+    }
   if (copy_xattr (inputname, outputname) != 0)
     {
       fclose (input);
@@ -452,7 +469,9 @@ loop_over_group_file (int quiet)
       strcpy (oldname, files_etc_dir);
       strcat (oldname, "/group.old");
       unlink (oldname);
-      link (inputname, oldname);
+      if (link (inputname, oldname) < 0)
+	fprintf (stderr,
+		 _("Warning: cannot create backup of group file: %m\n"));
       rename (outputname, inputname);
     }
   else
@@ -612,8 +631,28 @@ sort_group_file (void)
       retval = -1;
       goto error_group;
     }
-  fchmod (newgf_fd, group_stat.st_mode);
-  fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid);
+  if (fchmod (newgf_fd, group_stat.st_mode) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change mode for `%s': %s\n"),
+	       group_tmp, strerror (errno));
+      fclose (oldgf);
+      close (newgf_fd);
+      unlink (group_tmp);
+      retval = -1;
+      goto error_group;
+    }
+  if (fchown (newgf_fd, group_stat.st_uid, group_stat.st_gid) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change owner/group for `%s': %s\n"),
+	       group_tmp, strerror (errno));
+      fclose (oldgf);
+      close (newgf_fd);
+      unlink (group_tmp);
+      retval = -1;
+      goto error_group;
+    }
   if (copy_xattr (group_orig, group_tmp) != 0)
     {
       fclose (oldgf);
@@ -679,7 +718,9 @@ sort_group_file (void)
       goto error_group;
     }
   unlink (group_old);
-  link (group_orig, group_old);
+  if (link (group_orig, group_old) < 0)
+    fprintf (stderr,
+	     _("Warning: cannot create backup of group file: %m\n"));
   rename (group_tmp, group_orig);
  error_group:
   unlink (group_tmp);
@@ -698,8 +739,6 @@ main (int argc, char *argv[])
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
 
-  openlog (program, LOG_PID, LOG_AUTHPRIV);
-
   while (1)
     {
       int c;
@@ -737,7 +776,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
