@@ -67,7 +67,7 @@
 /* How often should we try to lock the passwd database ?  */
 #define MAX_LOCK_RETRIES 3
 
-static const struct pam_conv conv = {
+static struct pam_conv conv = {
   misc_conv,
   NULL
 };
@@ -129,12 +129,43 @@ display_pw (const struct passwd *pw)
     printf ("%s %s\n", pw->pw_name, pw_status (pw->pw_passwd));
 }
 
+/* A conversation function which uses an internally-stored value for
+ * the responses. */
+static int
+stdin_conv (int num_msg, const struct pam_message **msgm,
+	    struct pam_response **response, void *appdata_ptr)
+{
+  struct pam_response *reply;
+  int count;
+
+  /* Sanity test. */
+  if (num_msg <= 0)
+    return PAM_CONV_ERR;
+
+  /* Allocate memory for the responses. */
+  reply = calloc (num_msg, sizeof (struct pam_response));
+  if (reply == NULL)
+    return PAM_CONV_ERR;
+
+  /* Each prompt elicits the same response. */
+  for (count = 0; count < num_msg; ++count)
+    {
+      reply[count].resp_retcode = 0;
+      reply[count].resp = strdup (appdata_ptr);
+    }
+
+  /* Set the pointers in the response structure and return. */
+  *response = reply;
+  return PAM_SUCCESS;
+}
+
 static void
 print_usage (FILE *stream, const char *program)
 {
-  fprintf (stream, _("Usage: %s [-f|-g|-s|-k[-q]] [name]\n"), program);
-  fprintf (stream, _("       %s [-D binddn] [-n min] [-x max] [-w warn] [-i inact] user\n"), program);
-  fprintf (stream, _("       %s {-l|-u|-d|-S[-a]|-e|-h} user\n"), program);
+  fprintf (stream, _("Usage: %s [-f|-g|-s|-k[-q]] [account]\n"), program);
+  fprintf (stream, _("       %s [-D binddn] [-n min] [-x max] [-w warn] [-i inact] account\n"), program);
+  fprintf (stream, _("       %s {-l|-u|-d|-S[-a]|-e|-h} account\n"), program);
+  fprintf (stream, _("       %s --stdin account\n"), program);
 }
 
 static void
@@ -153,17 +184,18 @@ print_help (const char *program)
   fputs (_("  -S             Show the password attributes\n"), stdout);
   fputs (_("  -a             Only with -S, show for all accounts\n"), stdout);
   fputs (_("  -d             Delete the password for the named account\n"), stdout);
-  fputs (_("  -l             Locks the password entry for \"user\"\n"), stdout);
-  fputs (_("  -u             Try to unlock the password entry for \"user\"\n"), stdout);
+  fputs (_("  -l             Locks the password entry for \"account\"\n"), stdout);
+  fputs (_("  -u             Try to unlock the password entry for \"account\"\n"), stdout);
   fputs (_("  -e             Force the user to change password at next login\n"), stdout);
-  fputs (_("  -n min         Set minimum field for \"user\"\n"), stdout);
-  fputs (_("  -x max         Set maximum field for \"user\"\n"), stdout);
-  fputs (_("  -w warn        Set warn field for \"user\"\n"), stdout);
+  fputs (_("  -n min         Set minimum field for \"account\"\n"), stdout);
+  fputs (_("  -x max         Set maximum field for \"account\"\n"), stdout);
+  fputs (_("  -w warn        Set warn field for \"account\"\n"), stdout);
   fputs (_("  -r service     Use nameservice 'service'\n"), stdout);
   fputs (_("  -D binddn      Use dn \"binddn\" to bind to the LDAP directory\n"),
 	 stdout);
   fputs (_("  -P path        Search passwd and shadow file in \"path\"\n"),
 	 stdout);
+  fputs (_("  --stdin        Read new password from stdin (root only)\n"), stdout);
   fputs (_("  --help         Give this help list\n"), stdout);
   fputs (_("  --usage        Give a short usage message\n"), stdout);
   fputs (_("  --version      Print program version\n"), stdout);
@@ -186,7 +218,7 @@ passwd_main (const char *program, int argc, char **argv)
   char *binddn = NULL;
   int k_flag = 0, a_flag = 0, d_flag = 0, e_flag = 0, h_flag = 0,
     i_flag = 0, l_flag = 0, n_flag = 0, u_flag = 0, x_flag = 0,
-    S_flag = 0, w_flag = 0, P_flag = 0;
+    S_flag = 0, w_flag = 0, P_flag = 0, stdin_flag = 0;
   long inact = 0, age_min = 0, age_max = 0, warn = 0;
 
   /* Parse program arguments */
@@ -202,6 +234,7 @@ passwd_main (const char *program, int argc, char **argv)
 	  {"version", no_argument, NULL, '\255'},
 	  {"usage", no_argument, NULL, '\254'},
 	  {"help", no_argument, NULL, '\253'},
+	  {"stdin", no_argument, NULL, '\252'},
 	  {NULL, 0, NULL, '\0'}
 	};
 
@@ -313,6 +346,9 @@ passwd_main (const char *program, int argc, char **argv)
 	  P_flag = 1;
 	  files_etc_dir = strdup (optarg);
 	  break;
+	case '\252':
+	  stdin_flag = 1;
+	  break;
 	case '\253':
           print_help (program);
           return 0;
@@ -372,6 +408,13 @@ passwd_main (const char *program, int argc, char **argv)
       return E_SUCCESS;
     }
 
+  if (stdin_flag && uid != 0)
+    {
+      fprintf (stderr, _("%s: Permission denied.\n"), program);
+      sec_log (program, MSG_STDIN_FOR_NONROOT_DENIED, uid);
+      return E_NOPERM;
+    }
+
   if (S_flag && (admin_only || k_flag))
     {
       print_error (program);
@@ -514,6 +557,30 @@ passwd_main (const char *program, int argc, char **argv)
 	  return E_USAGE;
 	}
 
+
+      if (stdin_flag)
+	{
+	  char *ptr;
+	  char password[160]; /* 127 is the longest with current crypt */
+	  int i;
+
+	  i = read (STDIN_FILENO, password, sizeof (password) - 1);
+	  if (i < 0)
+	    {
+	      fprintf (stderr, _("%s: error reading from stdin!\n"),
+		       program);
+	      return E_FAILURE;
+	    }
+
+	  password[i] = '\0';
+	  /* Remove trailing \n.  */
+	  ptr = strchr (password, '\n');
+	  if (ptr)
+	    *ptr = 0;
+	  conv.conv = stdin_conv;
+	  conv.appdata_ptr = strdup (password);
+	}
+
       if (!silent)
 	printf (_("Changing password for %s.\n"), pw->pw_name);
 
