@@ -26,7 +26,6 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <syslog.h>
 #include <getopt.h>
 #include <locale.h>
 #include <libintl.h>
@@ -270,8 +269,26 @@ loop_over_passwd_file (int quiet)
       fclose (input);
       return E_NO_FILE;
     }
-  fchmod (output_fd, passwd_stat.st_mode);
-  fchown (output_fd, passwd_stat.st_uid, passwd_stat.st_gid);
+  if (fchmod (output_fd, passwd_stat.st_mode) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change mode for `%s': %s\n"),
+	       outputname, strerror (errno));
+      fclose (input);
+      close (output_fd);
+      unlink (outputname);
+      return E_NO_FILE;
+    }
+  if (fchown (output_fd, passwd_stat.st_uid, passwd_stat.st_gid) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change owner/group for `%s': %s\n"),
+	       outputname, strerror (errno));
+      fclose (input);
+      close (output_fd);
+      unlink (outputname);
+      return E_NO_FILE;
+    }
   if (copy_xattr (inputname, outputname) != 0)
     {
       fclose (input);
@@ -446,7 +463,9 @@ loop_over_passwd_file (int quiet)
       strcpy (oldname, files_etc_dir);
       strcat (oldname, "/passwd.old");
       unlink (oldname);
-      link (inputname, oldname);
+      if (link (inputname, oldname) < 0)
+	fprintf (stderr,
+		 _("Warning: cannot create backup of passwd file: %m\n"));
       rename (outputname, inputname);
     }
   else
@@ -512,8 +531,26 @@ loop_over_shadow_file (int quiet)
       fclose (input);
       return E_NO_FILE;
     }
-  fchmod (output_fd, shadow_stat.st_mode);
-  fchown (output_fd, shadow_stat.st_uid, shadow_stat.st_gid);
+  if (fchmod (output_fd, shadow_stat.st_mode) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change mode for `%s': %s\n"),
+	       outputname, strerror (errno));
+      fclose (input);
+      close (output_fd);
+      unlink (outputname);
+      return E_NO_FILE;
+    }
+  if (fchown (output_fd, shadow_stat.st_uid, shadow_stat.st_gid) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change owner/group for `%s': %s\n"),
+	       outputname, strerror (errno));
+      fclose (input);
+      close (output_fd);
+      unlink (outputname);
+      return E_NO_FILE;
+    }
   if (copy_xattr (inputname, outputname) != 0)
     {
       fclose (input);
@@ -685,7 +722,9 @@ loop_over_shadow_file (int quiet)
       strcpy (oldname, files_etc_dir);
       strcat (oldname, "/shadow.old");
       unlink (oldname);
-      link (inputname, oldname);
+      if (link (inputname, oldname) < 0)
+	fprintf (stderr,
+		 _("Warning: cannot create backup of shadow file: %m\n"));
       rename (outputname, inputname);
     }
   else
@@ -845,8 +884,28 @@ sort_passwd_file (void)
       retval = -1;
       goto error_passwd;
     }
-  fchmod (newgf_fd, passwd_stat.st_mode);
-  fchown (newgf_fd, passwd_stat.st_uid, passwd_stat.st_gid);
+  if (fchmod (newgf_fd, passwd_stat.st_mode) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change mode for `%s': %s\n"),
+	       passwd_tmp, strerror (errno));
+      fclose (oldgf);
+      close (newgf_fd);
+      unlink (passwd_tmp);
+      retval = -1;
+      goto error_passwd;
+    }
+  if (fchown (newgf_fd, passwd_stat.st_uid, passwd_stat.st_gid) < 0)
+    {
+      fprintf (stderr,
+	       _("Cannot change owner/group for `%s': %s\n"),
+	       passwd_tmp, strerror (errno));
+      fclose (oldgf);
+      close (newgf_fd);
+      unlink (passwd_tmp);
+      retval = -1;
+      goto error_passwd;
+    }
   if (copy_xattr (passwd_orig, passwd_tmp) != 0)
     {
       fclose (oldgf);
@@ -912,7 +971,9 @@ sort_passwd_file (void)
       goto error_passwd;
     }
   unlink (passwd_old);
-  link (passwd_orig, passwd_old);
+  if (link (passwd_orig, passwd_old) < 0)
+    fprintf (stderr,
+	     _("Warning: cannot create backup of passwd file: %m\n"));
   rename (passwd_tmp, passwd_orig);
  error_passwd:
   unlink (passwd_tmp);
@@ -931,8 +992,6 @@ main (int argc, char *argv[])
   bindtextdomain(PACKAGE, LOCALEDIR);
   textdomain(PACKAGE);
 
-  openlog (program, LOG_PID, LOG_AUTHPRIV);
-
   while (1)
     {
       int c;
@@ -970,7 +1029,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
