@@ -1,4 +1,4 @@
-/* Copyright (C) 2004 Thorsten Kukuk
+/* Copyright (C) 2004, 2005 Thorsten Kukuk
    Author: Thorsten Kukuk <kukuk@suse.de>
 
    This program is free software; you can redistribute it and/or modify
@@ -131,7 +131,7 @@ main (int argc, char *argv[])
           print_help (program);
           return 0;
         case 'v':
-          print_version (program, "2004");
+          print_version (program, "2005");
           return 0;
         case 'u':
           print_usage (stdout, program);
@@ -191,7 +191,12 @@ main (int argc, char *argv[])
 	  fputs ("running out of memory!\n", stderr);
 	  return E_FAILURE;
 	}
-      link (path, tmppasswd);
+      if (link (path, tmppasswd) < 0)
+	{
+	  fprintf (stderr, _("Cannot create backup file `%s': %m\n"),
+		   tmppasswd);
+	  return E_FAILURE;
+	}
       free (path);
     }
 
