@@ -164,6 +164,7 @@ struct default_t
   char *skel;
   unsigned int groupscnt;
   char **groups;
+  int create_mail_spool;
 };
 
 /* This function converts a comma seperated list of groups
@@ -432,7 +433,21 @@ no integer value >= -1 in %s.\n"), program, &cp[7], configfile);
 	      ++errors;
 	    }
 	  continue;
-	}
+        }
+      else if (strncasecmp (cp, "CREATE_MAIL_SPOOL=", 18) == 0)
+        {
+	  if (strcasecmp (&cp[18], "yes") == 0)
+            data->create_mail_spool = 1;
+          else if (strcasecmp (&cp[18], "no") == 0)
+            data->create_mail_spool = 0;
+          else
+            {
+              ++errors;
+              fprintf (stderr, _("%s: Invalid value `%s' for option CREATE_MAIL_SPOOL in %s.\n"),
+                       program, &cp[18], configfile);
+            }
+          continue;
+        }
     }
   fclose (fp);
 
@@ -465,6 +480,8 @@ print_defaults (const struct default_t *data)
       printf ("%s", data->groups[i]);
     }
   printf ("\n");
+
+  printf ("CREATE_MAIL_SPOOL=%s\n", data->create_mail_spool?"yes":"no");
 }
 
 /* Write the config file (/etc/default/useradd)  */
@@ -480,7 +497,7 @@ write_defaults (const char *configfile, struct default_t *data)
   int new_fd, ret;
   int group_written = 0, home_written = 0, inactive_written = 0;
   int expire_written = 0, shell_written = 0, groups_written = 0;
-  int skel_written = 0;
+  int skel_written = 0, create_mail_spool_written = 0;
 
   fp = fopen (configfile, "r");
   if (NULL == fp)
@@ -507,8 +524,8 @@ write_defaults (const char *configfile, struct default_t *data)
   if (fchmod (new_fd, oldmode.st_mode) < 0)
     {
       fprintf (stderr,
-	       _("Cannot change mode for `%s': %s\n"),
-	       tmpname, strerror (errno));
+	       _("%s: Cannot change mode for `%s': %s\n"),
+	       program, tmpname, strerror (errno));
       fclose (fp);
       close (new_fd);
       unlink (tmpname);
@@ -517,8 +534,8 @@ write_defaults (const char *configfile, struct default_t *data)
   if (fchown (new_fd, oldmode.st_uid, oldmode.st_gid) < 0)
     {
       fprintf (stderr,
-	       _("Cannot change owner/group for `%s': %s\n"),
-	       tmpname, strerror (errno));
+	       _("%s: Cannot change owner/group for `%s': %s\n"),
+	       program, tmpname, strerror (errno));
       fclose (fp);
       close (new_fd);
       unlink (tmpname);
@@ -638,6 +655,13 @@ write_defaults (const char *configfile, struct default_t *data)
 	  groups_written = 1;
 	  continue;
 	}
+      else if (strncasecmp (cp, "CREATE_MAIL_SPOOL=", 18) == 0)
+        {
+          fprintf (new_fp, "CREATE_MAIL_SPOOL=%s\n",
+		   data->create_mail_spool?"yes":"no");
+          create_mail_spool_written = 1;
+          continue;
+        }
 
       fputs (buf, new_fp);
     }
@@ -675,6 +699,9 @@ write_defaults (const char *configfile, struct default_t *data)
 	}
       fputs ("\n", new_fp);
     }
+  if (!create_mail_spool_written)
+    fprintf (new_fp, "CREATE_MAIL_SPOOL=%s\n",
+	     data->create_mail_spool?"yes":"no");
 
   /* Close files.  */
   fclose (fp);
@@ -894,8 +921,111 @@ add_gr_mem (const char *name, char **gr_mem)
   return groups;
 }
 
+/* Create the users mail spool file if it does not exist. The
+   permissions we be 600, if the mail directory is worldwide writeable,
+   else 660 onwed by the user and group 'mail'. */
+static int
+create_mail_file (const char *user, uid_t uid, gid_t user_gid)
+{
+  int mode = 0600;
+  gid_t gid = user_gid;
+  int fd;
+  char *fname;
+  struct stat st;
+  char *cp;
+
+  if ((fname = malloc (strlen (user) +
+		       strlen (_PATH_MAILDIR) + 2)) == NULL)
+    {
+      fputs ("running out of memory!\n", stderr);
+      return E_MAIL_SPOOL;
+    }
+  cp = stpcpy (fname, _PATH_MAILDIR);
+  *cp++ = '/';
+  strcpy (cp, user);
+
+  if (access (fname, R_OK) == 0)
+    return 0;
+
+  if (stat (_PATH_MAILDIR, &st) == -1)
+    {
+      fprintf (stderr, _("%s: Can't stat %s: %m\n"),
+	       program, _PATH_MAILDIR);
+      free (fname);
+      return E_MAIL_SPOOL;
+    }
+
+  /* if directory is not worldwide writeable, the new file needs
+     write permissions for group 'mail'.  */
+  if ((st.st_mode & S_IWOTH) != S_IWOTH)
+    {
+      int buflen = 256;
+      char *buffer = alloca (buflen);
+      struct group resultbuf;
+      struct group *gr;
+
+      while (getgrnam_r ("mail", &resultbuf, buffer, buflen, &gr) != 0
+	     && errno == ERANGE)
+	{
+	  errno = 0;
+	  buflen += 256;
+	  buffer = alloca (buflen);
+	}
+
+      /* Only writeable for group 'mail', if this group exists.  */
+      if (gr)
+	{
+	  gid = gr->gr_gid;
+	  mode = 0660;
+	}
+      else
+	fprintf (stderr,
+		 _("%s: No group named \"mail\" exists, creating mail spool with mode 0600.\n"),
+		 program);
+    }
+
+  fd = open (fname, O_CREAT|O_EXCL|O_WRONLY|O_TRUNC, 0);
+  if (fd == -1)
+    {
+      fprintf (stderr,
+	       _("%s: Can't create mail spool for user %s.\n"),
+	       program, user);
+      free (fname);
+      return E_MAIL_SPOOL;
+    }
+  else
+    {
+      if (fchown (fd, uid, gid) < 0)
+	{
+	  fprintf (stderr,
+		   _("%s: Cannot change owner/group for `%s': %s\n"),
+		   program, fname, strerror (errno));
+	  unlink (fname);
+	  free (fname);
+	  close (fd);
+	  return E_MAIL_SPOOL;
+	}
+      if (fchmod (fd, mode) < 0)
+	{
+	  fprintf (stderr,
+		   _("%s: Cannot change mode for `%s': %s\n"),
+		   program, fname, strerror (errno));
+	  unlink (fname);
+	  free (fname);
+	  close (fd);
+	  return E_MAIL_SPOOL;
+	}
+      close (fd);
+    }
+
+  free (fname);
+
+  return 0;
+}
+
+
 /* Create the users home directory if it does not exist. The
-   permissions will calculated from UMASK in /etc/login.defs.  */
+   permissions will be calculated from UMASK in /etc/login.defs.  */
 static int
 create_home_directory (const char *home, uid_t uid, gid_t gid,
 		       const char *skeldir)
@@ -943,14 +1073,14 @@ create_home_directory (const char *home, uid_t uid, gid_t gid,
       if (chown (home, uid, gid) < 0)
 	{
 	  fprintf (stderr,
-		   _("Cannot change owner/group for `%s': %s\n"),
-		   home, strerror (errno));
+		   _("%s: Cannot change owner/group for `%s': %s\n"),
+		   program, home, strerror (errno));
 	  retval = E_HOMEDIR;
 	}
       if (chmod (home, 0777 & ~getlogindefs_num ("UMASK", 077)) < 0)
 	{
-	  fprintf (stderr, _("Cannot change mode for `%s': %s\n"),
-		   home, strerror (errno));
+	  fprintf (stderr, _("%s: Cannot change mode for `%s': %s\n"),
+		   program, home, strerror (errno));
 	  retval = E_HOMEDIR;
 	}
     }
@@ -972,7 +1102,7 @@ int
 main (int argc, char **argv)
 {
   struct default_t dflt =
-    { 100, "/home", -1, -1, "/bin/bash", "/etc/skel", 0, NULL };
+    { 100, "/home", -1, -1, "/bin/bash", "/etc/skel", 0, NULL, 0 };
   struct option *long_options = long_options_all;
   const char *short_options = short_options_all;
   char *use_service = NULL;
@@ -1643,25 +1773,33 @@ main (int argc, char **argv)
       nscd_flush_cache ("group");
 #endif
 
+      if (use_service == NULL || strcmp (use_service, "files") == 0)
+	ulckpwdf ();
+
+      if (dflt.create_mail_spool)
+	{
+	  int ret = create_mail_file (pw_data.pw.pw_name,
+				      pw_data.pw.pw_uid,
+				      pw_data.pw.pw_gid);
+	  if (ret != 0)
+	    return ret;
+	  sec_log (program, MSG_MAIL_FILE_CREATED,
+		   pw_data.pw.pw_name, getuid ());
+	}
+
       if (create_homedir)
 	{
 	  int ret = create_home_directory (new_home, pw_data.pw.pw_uid,
 					   pw_data.pw.pw_gid,
 					   skeldir ? : dflt.skel);
 	  if (ret != 0)
-	    {
-	      if (use_service == NULL || strcmp (use_service, "files") == 0)
-		ulckpwdf ();
-	      return ret;
-	    }
+	    return ret;
+
 	  sec_log (program, MSG_HOME_DIR_CREATED,
 		   pw_data.pw.pw_name, pw_data.pw.pw_uid, new_home,
 		   getuid ());
 	}
 
-      if (use_service == NULL || strcmp (use_service, "files") == 0)
-	ulckpwdf ();
-
       i = call_script ("USERADD_CMD", pw_data.pw.pw_name, pw_data.pw.pw_uid,
 		       pw_data.pw.pw_gid, pw_data.pw.pw_dir, program);
       if (i != 0)
